package test.infinispan;

import java.util.concurrent.Callable;
import java.util.concurrent.CyclicBarrier;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

import org.infinispan.client.hotrod.Flag;
import org.infinispan.client.hotrod.RemoteCache;
import org.infinispan.client.hotrod.RemoteCacheManager;
import org.infinispan.client.hotrod.VersionedValue;

public class Client implements Callable<Void>{
	
	private long start;
	private long end;
	private CyclicBarrier b;
	private RemoteCache<String, String> cache;
	private AtomicLong c;
	private RemoteCache<String, String> cache2;

	public Client(long start, long end, CyclicBarrier b, RemoteCache<String, String> cache, RemoteCache<String, String> cache2, AtomicLong c) {
		this.start = start;
		this.end = end;
		this.b = b;
		this.cache = cache;
		this.cache2 = cache2;
		this.c = c;
	}

	public Void call() throws Exception
	{
		b.await();
		
		for(long i = start; i < end; i++)
		{
			try
			{
				cache2.putIfAbsent(String.valueOf(i), String.valueOf(i));
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			
			for(int k = 0; k < 3; k++)
			{
				try
				{
					String key = String.valueOf(i);
					
					VersionedValue<String> versioned = cache.getVersioned(key);
					if(versioned == null)
					{
						String value = i + ":" + k + "@";
						cache.withFlags(Flag.FORCE_RETURN_VALUE).putIfAbsent(key, value);
					}
					else
					{
						String value = versioned.getValue() + i + ":" + k + "@";
						cache.withFlags(Flag.FORCE_RETURN_VALUE).replaceWithVersion(key, value, versioned.getVersion());
					}
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			
			c.incrementAndGet();
		}
		
		return null;
	}
	
	public static void main(String[] args) throws InterruptedException {
		RemoteCacheManager remote = new RemoteCacheManager(System.getProperties());
		RemoteCache<String, String> cache = remote.getCache("communication");
		RemoteCache<String, String> cache2 = remote.getCache("customer");
		
		int numThreads = args.length > 0 ? Integer.parseInt(args[0]) : 64;
		long numRequests = 1000000;
		
		long numReqPerThread = numRequests / numThreads;
		long rest = numRequests % numThreads;

		ExecutorService exec = Executors.newFixedThreadPool(numThreads);
		ExecutorCompletionService<Void> cs = new ExecutorCompletionService<Void>(exec);
		
		System.out.println("Starting with " + numThreads + " threads");
		
		AtomicLong c = new AtomicLong();
		CyclicBarrier b = new CyclicBarrier(numThreads);
		for(int i = 0; i < numThreads; i++)
		{
			long start = i * numReqPerThread;
			long end = (i + 1) * numReqPerThread;
			if(i == numThreads - 1)
			{
				end += rest;
			}
			
			cs.submit(new Client(start, end, b, cache, cache2, c));
		}
		
		while(c.get() < numRequests)
		{
			System.out.print("\r" + c.get());
			Thread.sleep(1000);
		}
		
		
		System.out.println("Done");
		exec.shutdown();
		exec.awaitTermination(10, TimeUnit.MINUTES);
		
		System.out.println("Shutdown");
	}
}
